//-----------------------------------------------------------------------------
// EXT_MOD_ETH
// Version 3.0
// 17.12.2011 / UB
// "www.Mikrocontroller-4U.de"
// mc-4u(@)t-online.de
//-----------------------------------------------------------------------------
// Fuer ein externes Modul
// mit dem Ethernet-IC : ENC28J60
//-----------------------------------------------------------------------------
// das File muss im Ordner
// "myAVR/WinAVR/AVR/Include/UB_EXT_MODULE/"
// liegen und mit #include <UB_EXT_MODULE/EXT_MOD_ETH.h>
// eingebunden werden
//-----------------------------------------------------------------------------
// Belegt :  Port-B (Bit4) fuer ChipSelect
//           Port-B (Bit5) fuer Interrupt-Eingang    << optional
//           Port-B (Bit6) fuer Reset                << optional
// Benutzt : SPI = Port-B (Bit1 bis Bit3)
//-----------------------------------------------------------------------------
//
// Hinweis : das Modul muss am MK3-Board entweder an der linken oder rechten
//           Buchsenleiste angesteckt werden
//
//           Der ENC28J60 funktioniert nur im SPI_Mode_0 
//           die default SPI-Frq ist 2MHz (getestet bis SPI-Frq von 8MHz)
//           
// Patchplan :
//
// 4 bis 6 Leitungen muessen verdrahtet werden
// (in der Default-Einstellung liegt alles auf Port-B)
//
// Port-B1 (SPI-SCK)  muss zum ETH-Modul Pin 19  (ENC28J60 SCK)
// Port-B2 (SPI-MOSI) muss zum ETH-Modul Pin 18  (ENC28J60 SDI)
// Port-B3 (SPI-MISO) muss zum ETH-Modul Pin 17  (ENC28J60 SDO)
// Port-B4 (Chip-Sel) muss zum ETH-Modul Pin 16  (ENC28J60 CS)
// Port-B5 (PCInt5)   muss zum ETH-Modul Pin 15  (ENC28J60 INT)    << optional
// Port-B6 (PB6)      muss zum ETH-Modul Pin 12  (ENC28J60 Reset)  << optional
//
//-----------------------------------------------------------------------------



//----------------------------------------------------------------------------
// Includes
//----------------------------------------------------------------------------
#include "UB_ETH_LOLEVEL/ETH_LO_GLOBAL.h"   // Globale spezifikationen
#include "ub_mk3_2560/MK3_2560_SPI.h"       // bindet die SPI-Funktionen ein
#include "UB_ETH_LOLEVEL/ETH_LO_ENC28J60.h" // Funktionen fuer den ENC28J60
#include "UB_ETH_LOLEVEL/ETH_LO_OSI1_ETH.h" // Funktionen fuer die Ethernet-Protokolle




//-----------------------------------------------------------------------------
// INFOS zur Benutzung :
//-----------------------------------------------------------------------------

// -> Alle Settings (Hardware+Software) muessen im File
//    "ETH_LO_GLOBAL.h" vorgenommen werden



//-----------------------------------------------------------------------------
// Enumerationen fuer die Funktioen
//-----------------------------------------------------------------------------
// siehe ETH_LO_GLOBAL.h




//-----------------------------------------------------------------------------
// Funktionen die vom MAIN benutzt werden koennen
//-----------------------------------------------------------------------------
void EXT_INIT_ETH(void);
void EXT_ETH_START(void);
void EXT_ETH_RESET(void);
void EXT_ETH_POLL(void);
bool EXT_ETH_UDP_IN(char *ptr);
void EXT_ETH_UDP_OUT(const char *ptr);
bool EXT_ETH_CHECK_HOST(void);
//-----------------------------------------------------------------------------



//-----------------------------------------------------------------------------
// Port-Pin definition
//-----------------------------------------------------------------------------
// siehe ETH_LO_GLOBAL.h




//-----------------------------------------------------------------------------
void EXT_INIT_ETH(void) {
  // zum initialisieren 
  // des Ethernet-Moduls

  // init der Chip-Select-Leitung  
  MK3_DDR_ETH|=ENC28J60_CS;     // Chip-Select als Ausgang
  MK3_PORT_ETH|=ENC28J60_CS;    // Chip-Select AUS

  #if USE_ETH_INTMODE==1
    // init der Interrupt-Leitung
    MK3_DDR_ETH&=~ENC28J60_INT;   // Interrupt-Pin als Eingang
    MK3_PORT_ETH|=ENC28J60_INT;   // Pull-Up aktivieren

    // init der Interrupt-Register
    PCICR|=0x01;                  // PCINT 0-7 enable
    PCMSK0|=0x20;                 // PCINT5 enable
  #endif


  #if USE_ETH_HWRESET==1
    // init der Reset-Leitung
    MK3_DDR_ETH|=ENC28J60_RESET;  // Reset als Ausgang
    MK3_PORT_ETH|=ENC28J60_RESET; // Reset auf HI
  #endif

  // init vom SPI-BUS
  MK3_INIT_SPI();                 // @16MHz -> 1MHz SPI-Clk, Mode-0
  
  // SPI-FRQ Einstellung
  MK3_SPI_SET_FRQ(SPI_CLK_DIV_8); // SPI-FRQ = 2MHz

  // Reset vom ENC28J60
  EXT_ETH_RESET();

  //---------------------------------------
  // Init der Struktur mit default werten
  // siehe ETH_LO_GLOBAL.h
  //---------------------------------------
  ETH_Setting.own_mac[0]=DEF_ETH_MAC_0;
  ETH_Setting.own_mac[1]=DEF_ETH_MAC_1;
  ETH_Setting.own_mac[2]=DEF_ETH_MAC_2;
  ETH_Setting.own_mac[3]=DEF_ETH_MAC_3;
  ETH_Setting.own_mac[4]=DEF_ETH_MAC_4;
  ETH_Setting.own_mac[5]=DEF_ETH_MAC_5;
  ETH_Setting.own_ip[0]=DEF_ETH_IP_0;
  ETH_Setting.own_ip[1]=DEF_ETH_IP_1;
  ETH_Setting.own_ip[2]=DEF_ETH_IP_2;
  ETH_Setting.own_ip[3]=DEF_ETH_IP_3;
  ETH_Setting.own_udp_port=DEF_ETH_UDP_PORT;
  ETH_Setting.led_a=DEF_ENC_LED_MODE_A;
  ETH_Setting.led_b=DEF_ENC_LED_MODE_B;
  ETH_Setting.buffer=DEF_ENC_BUFFER_MODE;

  //---------------------------------------
  // Init der Struktur mit default werten
  //---------------------------------------
  ENC28J60.status_ok=false;
  ENC28J60.phid1=0;
  ENC28J60.phid2=0;
  ENC28J60.version=0;
  ENC28J60.rx_packete=0;
  ENC28J60.rx_unknonwn=0;
  ENC28J60.rx_error=0;
  ENC28J60.rx_free=0;
  ENC28J60.tx_packete=0;

  //---------------------------------------
  // Init der Struktur mit default werten
  //--------------------------------------- 
  HOST_ETH_Data.host_ok=false; 
  HOST_ETH_Data.host_mac[0]=0x00;
  HOST_ETH_Data.host_mac[1]=0x00;
  HOST_ETH_Data.host_mac[2]=0x00;
  HOST_ETH_Data.host_mac[3]=0x00;
  HOST_ETH_Data.host_mac[4]=0x00;
  HOST_ETH_Data.host_mac[5]=0x00;
  HOST_ETH_Data.host_ip[0]=DEF_HOST_IP_0;
  HOST_ETH_Data.host_ip[1]=DEF_HOST_IP_1;
  HOST_ETH_Data.host_ip[2]=DEF_HOST_IP_2;
  HOST_ETH_Data.host_ip[3]=DEF_HOST_IP_3;  

  //---------------------------------------
  // Init der Struktur mit default werten
  //---------------------------------------  
  RX_ETH_Data.eth_frame_len=0;
  RX_ETH_Data.status=0;
  RX_ETH_Data.raw_data[0]=0x00;
  RX_ETH_Data.typfield=0;    
  RX_ETH_Data.ethtyp=ETH_TYP_UNKNOWN;
  RX_ETH_Data.frame_len=0;
  RX_ETH_Data.z_mac[0]=0x00;
  RX_ETH_Data.ip_frame_len=0;
  RX_ETH_Data.ip_data_len=0;  
  RX_ETH_Data.udp_q_port=0;
  RX_ETH_Data.udp_z_port=0;
  RX_ETH_Data.udp_data_len=0;

  //---------------------------------------
  // Init der Struktur mit default werten
  //---------------------------------------  
  TX_ETH_Data.tx_typ=ETH_TX_NOTHING;
  TX_ETH_Data.ip_id_nr=0x0001;
  TX_ETH_Data.eth_tx_len=0;
  TX_ETH_Data.udp_data_len=0;


  // test vom ENC28J60
  if(ENC_checkID()==true) {
    // status und version beschreiben
    ENC28J60.status_ok=true;
    ENC28J60.version=ENC_checkVersion();
  }
  else {
    // ID vom ENC28J60 wurde nicht gefunden
    // Status auf FALSE setzen
    ENC28J60.status_ok=false;
    ENC28J60.version=0;
  }
}

//-----------------------------------------------------------------------------
void EXT_ETH_START(void) {
  // einschalten des ENC28J60
  // und setzen aller Notwendigen einstellungen
  //
  // zu beginn muss die Struktur "ETH_Setting" ausgefuellt sein


  if(ENC28J60.status_ok==true) {
    // nur wenn IC ok ist
    
    // einstellen der LEDs
    ENC_setLedMode(ETH_Setting.led_a,ETH_Setting.led_b);

    // einstellen der RAM-Puffer Aufteilung
    ENC_setBuffer(ETH_Setting.buffer);

    // einstellen der MAC-Adresse
    ENC_setMAC();
  
    // setzen aller restlichen Einstellungen
    ENC_initENC28J60();

    // enable vom Ethernet-RX
    ENC_rxEnable();

    #if USE_ETH_INTMODE==1
      // enable der Interrupts
      ENC_IntEnable();
    #endif

    // Global-Interrupt
    SREG|=0x80; // Alle Interrupts einschalten
  }
}

//-----------------------------------------------------------------------------
void EXT_ETH_RESET(void) {
  // Reset des ENC28J60
  // entweder per Hardware-Pin oder
  // per Software Kommando

  unsigned int n;

  #if USE_ETH_HWRESET==1
    //-------------------------
    // Hardware Reset
    //-------------------------

    MK3_PORT_ETH|=ENC28J60_RESET; // Reset auf HI

    for(n=0;n<10000;n++) {
      // kleine Pause
      __asm volatile ("nop \n");  // mini Pause (ca 5ms @16MHz)
    }

    MK3_PORT_ETH&=~ENC28J60_RESET; // Reset auf Lo

    for(n=0;n<10000;n++) {
      // kleine Pause
      __asm volatile ("nop \n");  // mini Pause (ca 5ms @16MHz)
    }

    MK3_PORT_ETH|=ENC28J60_RESET; // Reset auf HI
  #else
    //-------------------------
    // Software Reset
    //-------------------------

    for(n=0;n<10000;n++) {
      // kleine Pause
      __asm volatile ("nop \n");  // mini Pause (ca 5ms @16MHz)
    }

    ENC_softReset();

    for(n=0;n<10000;n++) {
      // kleine Pause
      __asm volatile ("nop \n");  // mini Pause (ca 5ms @16MHz)
    }

  #endif
}

//-----------------------------------------------------------------------------
void EXT_ETH_POLL(void) {
  // fuer den Polling-Betrieb notwendig
  #if USE_ETH_INTMODE!=1
    if(ENC28J60.status_ok==true) {
      // nur wenn IC ok ist
      ETH_RX_Packet();
    }
  #endif
}

//-----------------------------------------------------------------------------
bool EXT_ETH_UDP_IN(char *ptr) {
  // zum auslesen von einem UDP-String
  // rueckgabe von true, wenn etwas im puffer steht

  bool ret_wert=false;

  if(ENC28J60.status_ok==true) {
    // nur wenn IC ok ist  
    ret_wert=UDP_receivePacket(ptr);
  }

  return(ret_wert);
}

//-----------------------------------------------------------------------------
void EXT_ETH_UDP_OUT(const char *ptr) {
  // zum senden eines Strings per UDP

  if(ENC28J60.status_ok==true) {
    // nur wenn IC ok ist
    if(HOST_ETH_Data.host_ok==true) {
      // nur wenn die Daten vom Host bekannt sind

      // UDP-Packet erstellen
      UDP_sendPacket(ptr);
      if(TX_ETH_Data.udp_data_len>0) {
        // IP-Checksumme hinzufuegen
        IP_makeIPChecksum();
        // UDP-Packet senden
        ETH_TX_Packet();
      }
    }
  }
}

//-----------------------------------------------------------------------------
bool EXT_ETH_CHECK_HOST(void) {
  // zum suchen von einem Host (per ARP-Protokoll)
  // rueckgabe von true, wenn Host gefunden wurde
  //
  // IP-Adresse vom Host muss in "HOST_ETH_Data.host_ip" stehen
 
  bool ret_wert=false;
  unsigned int v_wait=0;  
  unsigned int n;

  if(ENC28J60.status_ok==true) {
    // nur wenn IC ok ist

    // zuerst auf false
    HOST_ETH_Data.host_ok=false;

    // ARP-Packet erstellen
    ARP_makeARPRequest();
    // ARP-Packet senden
    ETH_TX_Packet();

    // warten bis die Antwort
    // vom Host gesendet wurde
    // Wait_Timeout = ca 100ms

    #if USE_ETH_INTMODE==1
      // im Interrupt-Betrieb
      for(v_wait=0;v_wait<ETH_ARP_REQUEST_TIMEOUT_INT;v_wait++) {
        for(n=0;n<100;n++) {
          // kleine Pause
          __asm volatile ("nop \n");
        }		
        if(HOST_ETH_Data.host_ok==true) {
          // wenn HOST geantwortet hat
          ret_wert=true;
          break;
        }
      }
    #else
      // im Polling-Betrieb
      for(v_wait=0;v_wait<ETH_ARP_REQUEST_TIMEOUT_POLL;v_wait++) {
        for(n=0;n<100;n++) {
          // kleine Pause
          __asm volatile ("nop \n");
        }
        EXT_ETH_POLL();
        if(HOST_ETH_Data.host_ok==true) {
          // wenn HOST geantwortet hat
          ret_wert=true;
          break;
        }
      }
    #endif	
  }  

  return(ret_wert);
}
