//-----------------------------------------------------------------------------
// ETH_LO_OSI1_ETH
// Version 3.0
// 17.12.2011 / UB
// "www.Mikrocontroller-4U.de"
// mc-4u(@)t-online.de
//-----------------------------------------------------------------------------
// Lo-Level Funktionen
// mit dem Ethernet-IC : ENC28J60
//
// Funktionen : OSI-Schicht-1 implementation
//              Ethernet-Frame
//
//-----------------------------------------------------------------------------
// das File muss im Ordner
// "myAVR/WinAVR/AVR/Include/UB_EXT_MODULE/UB_ETH_LOLEVEL/"
// liegen und mit #include <UB_ETH_LOLEVEL/ETH_LO_OSI1_ETH.h>
// eingebunden werden
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
//
// ETH-Packet (Header)
// --------------------
// Laenge         = 14 Bytes (Prambel wird vom ENC28J60 ausgefiltert)
// Start im Array = ab Adresse 0
//
// Adr 00 - 05    = Ziel MAC-Adr (6 Bytes)
// Adr 06 - 11    = Quell MAC-Adr (6 Bytes)
// Adr 12 - 13    = Typ-Field (2 Bytes) [ARP,IP,usw]
//
// ETH-Packet (Daten)
// -------------------
// im Anschluss an den ETH-Header folgt
// der Frame des jeweiligen Typs (z.B. ARP oder IP usw)
//-----------------------------------------------------------------------------


#ifndef UB_ETH_ETH    // Wichtig !!
#define UB_ETH_ETH    // damit wird das File nur einmal eingebunden

//-----------------------------------------------------------------------------
// Funktionen die von extern benutzt werden koennen
//-----------------------------------------------------------------------------
unsigned int ETH_makeChecksum(unsigned int start, unsigned int laenge);


//----------------------------------------------------------------------------
// Includes
//----------------------------------------------------------------------------
#include <avr/interrupt.h>
#include "ETH_LO_OSI2_ARP.h" // fuer ARP-Protokolle
#include "ETH_LO_OSI3_IP.h"  // fuer IP-Protokolle



//-----------------------------------------------------------------------------
// Enumerationen fuer die Funktioen
//-----------------------------------------------------------------------------
// siehe ETH_LO_GLOBAL.h


//-----------------------------------------------------------------------------
// interne Funktionen
//-----------------------------------------------------------------------------
ETH_RX_STAT_t ETH_RX_Packet(void);
void ETH_TX_Packet(void);
void ETH_readETHTypField(void);
void ETH_readETHData(void);
bool ETH_executeETHPacket(void);
void ETH_executeAntwort(void);
bool ETH_checkStatusVector(void);


//-----------------------------------------------------------------------------
// interne Defines
//-----------------------------------------------------------------------------

#define  ETH_HEADER_LEN  14    // Anzahl der Bytes im fuer den Header

#define  ETH_POS_Z_MAC_ADR   0 // position im ETH-Header der Ziel-MAC (6 Bytes)
#define  ETH_POS_Q_MAC_ADR   6 // position im ETH-Header der Quell-MAC (6 Bytes)
#define  ETH_POS_TYPFIELD_H 12 // position im ETH-Header vom TypField (hi-byte)
#define  ETH_POS_TYPFIELD_L 13 // position im ETH-Header vom TypField (lo-byte)


//-----------------------------------------------------------------------------
void ETH_executeAntwort(void) {
  // sendet ein Antwort Packet

  if(TX_ETH_Data.tx_typ==ETH_TX_ARP_REPLY) {
    // ARP-Reply senden
    ETH_TX_Packet();
  }
  else if(TX_ETH_Data.tx_typ==ETH_TX_ICMP_REPLY) {
    // ICMP-Reply senden
    ETH_TX_Packet();
  }
  else if(TX_ETH_Data.tx_typ==ETH_TX_UDP_PACKET) {
    // UDP-Packet senden
    ETH_TX_Packet();
  }
}

//-----------------------------------------------------------------------------
bool ETH_executeETHPacket(void) {
  // bearbeiten des empfangenen Packetes
  // je nach Typfield
  //
  // rueckgabe von true, wenn das packet bearbeitet wurde

  bool ret_wert=false;

  if(RX_ETH_Data.ethtyp==ETH_TYP_ARP) {
    // ein ARP-Protokoll bearbeiten
    // laenge ueberpruefen
    if(RX_ETH_Data.eth_frame_len<=ETH_MAX_ARP_SIZE) {
      // ARP-Packet bearbeiten
      ret_wert=ARP_executeARPPacket();
    }
  }
  else if(RX_ETH_Data.ethtyp==ETH_TYP_IP) {
    // ein IP-Protokoll bearbeiten
    ret_wert=IP_executeIPPacket();
  }

  return(ret_wert);
}


//-----------------------------------------------------------------------------
void ETH_readETHData(void) {
  // auswerten des Datenfelds
  // aus dem empfangenen Packet

  //----------------------------------------
  // Ziel MAC-Adr auslesen und speichern
  //----------------------------------------
  RX_ETH_Data.z_mac[0]=RX_ETH_Data.raw_data[ETH_POS_Z_MAC_ADR+0];
  RX_ETH_Data.z_mac[1]=RX_ETH_Data.raw_data[ETH_POS_Z_MAC_ADR+1];
  RX_ETH_Data.z_mac[2]=RX_ETH_Data.raw_data[ETH_POS_Z_MAC_ADR+2];
  RX_ETH_Data.z_mac[3]=RX_ETH_Data.raw_data[ETH_POS_Z_MAC_ADR+3];
  RX_ETH_Data.z_mac[4]=RX_ETH_Data.raw_data[ETH_POS_Z_MAC_ADR+4];
  RX_ETH_Data.z_mac[5]=RX_ETH_Data.raw_data[ETH_POS_Z_MAC_ADR+5];

  //----------------------------------------
  // Quell MAC-Adr auslesen und speichern
  //----------------------------------------
  HOST_ETH_Data.host_mac[0]=RX_ETH_Data.raw_data[ETH_POS_Q_MAC_ADR+0];
  HOST_ETH_Data.host_mac[1]=RX_ETH_Data.raw_data[ETH_POS_Q_MAC_ADR+1];
  HOST_ETH_Data.host_mac[2]=RX_ETH_Data.raw_data[ETH_POS_Q_MAC_ADR+2];
  HOST_ETH_Data.host_mac[3]=RX_ETH_Data.raw_data[ETH_POS_Q_MAC_ADR+3];
  HOST_ETH_Data.host_mac[4]=RX_ETH_Data.raw_data[ETH_POS_Q_MAC_ADR+4];
  HOST_ETH_Data.host_mac[5]=RX_ETH_Data.raw_data[ETH_POS_Q_MAC_ADR+5];
}

//-----------------------------------------------------------------------------
void ETH_readETHTypField(void) {
  // auswerten des TypFields
  // aus dem empfangenen Packet

  unsigned int uint=0;
  unsigned char lo,hi;

  //----------------------------------------
  // Typ-Field auslesen und speichern
  //----------------------------------------
  hi=RX_ETH_Data.raw_data[ETH_POS_TYPFIELD_H];
  lo=RX_ETH_Data.raw_data[ETH_POS_TYPFIELD_L];

  uint=(hi<<8);
  uint=(uint|lo);

  RX_ETH_Data.typfield=uint;

  //----------------------------------------
  // Typ-Field auswerten
  //----------------------------------------
  RX_ETH_Data.ethtyp=ETH_TYP_UNKNOWN;
  RX_ETH_Data.frame_len=RX_ETH_Data.eth_frame_len;
  if(RX_ETH_Data.typfield==0x0800) {
    // es wurde ein IP-Protokoll empfangen
    RX_ETH_Data.ethtyp=ETH_TYP_IP;       // IP-Packet
    RX_ETH_Data.frame_len=RX_ETH_Data.eth_frame_len;
  }
  else if(RX_ETH_Data.typfield==0x0806) {
    // es wurde ein ARP-Protokoll empfangen
    RX_ETH_Data.ethtyp=ETH_TYP_ARP;      // ARP-Packet
    RX_ETH_Data.frame_len=ARP_FRAME_LEN; // Laenge des kompletten ARP-Packets
  }
}

//-----------------------------------------------------------------------------
void ETH_TX_Packet(void) {
  // ein Packet absenden
  // die Lanege muss in "TX_ETH_Data.eth_tx_len" stehen

  unsigned char lo,hi,wert;
  unsigned int uint,n;
  unsigned int laenge=0;

  laenge=TX_ETH_Data.eth_tx_len;
  if(laenge>0) {
    ENC28J60.tx_packete++;

    // EWRPT,Bank0 setzen
    ENC_setControlRegisterBank(0);  
    lo=(ENC_TX_START&0x00FF);
    hi=(ENC_TX_START>>8);
    ENC_writeControlRegister(ENC28J60_CRE_EWRPTL,lo);
    ENC_writeControlRegister(ENC28J60_CRE_EWRPTH,hi);

    // ETXND,Bank0 setzen
    uint=ENC_TX_START+laenge;
    lo=(uint&0x00FF);
    hi=(uint>>8);  
    ENC_writeControlRegister(ENC28J60_CRE_ETXNDL,lo);
    ENC_writeControlRegister(ENC28J60_CRE_ETXNDH,hi);

    // den kompletten Block in den Buffer schreiben
    ENC_writeBufferBlock(laenge);

    // Packet per Ethernet senden
    wert=0;
    wert|=0x08; // TXRTS Bit setzen
    ENC_setBitRegister(ENC28J60_CRE_ECON1,wert);
  }
}

//-----------------------------------------------------------------------------
ETH_RX_STAT_t ETH_RX_Packet(void) {
  // test ob ein Packet empfangen wurde
  // Datasheet Seite 43
  // -> wird vom Interrupt aufgerufen
  //    oder von "EXT_ETH_POLL()"
  // 
  // es wird auch ein Fehler beim
  // empfang erkannt 
  //
  // rueckgabe, ob ein packet empfangen wurde
  
  ETH_RX_STAT_t ret_wert=ETH_RX_NO_PACKET;
  unsigned char wert;
  unsigned char lo,hi;
  unsigned int uint;
  unsigned char v_eir_flags;

  //--------------------------------------------------
  // zuerst Antwortflag zuruecksetzen
  //--------------------------------------------------
  TX_ETH_Data.tx_typ=ETH_TX_NOTHING;

  //--------------------------------------------------
  // test ob etwas empfangen wurde
  // PKTIF BIT in EIR auslesen
  //--------------------------------------------------
  v_eir_flags=ENC_readControlRegisterE(ENC28J60_CRE_EIR);
  if((v_eir_flags&0x40)!=0) {
    // wenn etwas empfangen wurde   

    #if USE_ETH_CALC_RX_FREE==1
      // freien Speicherplatz ausrechnen
      ENC28J60.rx_free=ENC_getFreeRXPuffer();
    #endif

    // Read Pointer fr RX einstellen
    lo=(ENC_RX_POINTER&0x00FF);
    hi=(ENC_RX_POINTER>>8);
    // ERDPT,Bank0 beschreiben
    ENC_setControlRegisterBank(0);    
    ENC_writeControlRegister(ENC28J60_CRE_ERDPTL,lo);
    ENC_writeControlRegister(ENC28J60_CRE_ERDPTH,hi);

    // Init-Werte auslesen
    ENC_readBufferBlock(0,0); 

    if(RX_ETH_Data.eth_frame_len>ETH_MAX_PACKET_SIZE) {
      // wenn packet groesser als Puffer im RAM
      // Packet verwerfen
      ret_wert=ETH_RX_SIZE_ERR;
      ENC28J60.rx_error++;
      RX_ETH_Data.typfield=0;
      RX_ETH_Data.ethtyp=ETH_TYP_UNKNOWN;
    }
    else if(ETH_checkStatusVector()==true) {
      // wenn der Status-Vektor ok war
      // zuerst den ETH-Header auslesen
      ENC_readBufferBlock(1,ETH_HEADER_LEN);
      // ETH-Daten auslesen
      ETH_readETHData();
      // Typ-Field auslesen
      ETH_readETHTypField();

      if(RX_ETH_Data.ethtyp!=ETH_TYP_UNKNOWN) {
        // falls es ein bekanntes Protokoll ist,
        // den Rest vom ETH-Packet auslesen

        // ETH-Footer auslesen
        ENC_readBufferBlock(2,ETH_HEADER_LEN);
        // jetzt das Packet bearbeiten
        if(ETH_executeETHPacket()==true) {
          // wenn das Packet bearbeitet wurde
          ret_wert=ETH_RX_OK;
          ENC28J60.rx_packete++;
          if(HOST_ETH_Data.host_ok==false) {
            // Daten vom Host sind jetzt bekannt (zum senden)
            HOST_ETH_Data.host_ok=true;
          } 
        }
        else {
          // wenn das Packet nicht bearbeitet wurde
          ret_wert=ETH_RX_NOT_FOR_ME;
          ENC28J60.rx_unknonwn++;
        }		
      }
      else {
        // wenn Typ nicht unterstuetzt wird
        // Packet verwerfen
        ret_wert=ETH_RX_UNKNOWN;
        ENC28J60.rx_unknonwn++;
      }
    }
    else {
      // Status-Vector hat einen Fehler gefunden
      // Packet verwerfen
      ret_wert=ETH_RX_STAT_ERR;
      ENC28J60.rx_error++;
      RX_ETH_Data.typfield=0;
      RX_ETH_Data.ethtyp=ETH_TYP_UNKNOWN;      
    }

    //-----------------------------------------------
    // VORSICHT !! das hier ist ein Workaround
    // aus dem Errata-Sheet Punkt.14 (Seite 6) 
    // -> es duerfen nur ungerade Adressen in 
    //    ERXRDPT geschrieben werden
    //-----------------------------------------------

    // ERXST,Bank0 auslesen
    ENC_setControlRegisterBank(0);
    lo=ENC_readControlRegisterE(ENC28J60_CRE_ERXSTL);
    hi=ENC_readControlRegisterE(ENC28J60_CRE_ERXSTH);
    uint=(hi<<8);
    uint=(uint|lo);

    // Pointer fuer RX neu einstellen
    // und damit den Speicher wieder freigeben
    if(ENC_RX_POINTER==ENC_RX_START) {
      // Pointer fr RX ganz normal einstellen
      lo=(ENC_RX_ENDE&0x00FF);
      hi=(ENC_RX_ENDE>>8);
      ENC_writeControlRegister(ENC28J60_CRE_ERXRDPTL,lo);
      ENC_writeControlRegister(ENC28J60_CRE_ERXRDPTH,hi);      
    }
    else {
      uint=(ENC_RX_POINTER-1);
      lo=(uint&0x00FF);
      hi=(uint>>8);
      ENC_writeControlRegister(ENC28J60_CRE_ERXRDPTL,lo);
      ENC_writeControlRegister(ENC28J60_CRE_ERXRDPTH,hi); 
    }

    // Packet Counter um 1 decrementieren
    wert=0;
    wert|=0x40; // PKTDEC Bit setzen
    ENC_setBitRegister(ENC28J60_CRE_ECON2,wert);
  }
  else {
    // wenn nichts empfangen wurde
    RX_ETH_Data.eth_frame_len=0;
    ret_wert=ETH_RX_NO_PACKET;
  }

  //--------------------------------------------------
  // test ob fehler beim Empfang
  // RXERIF BIT in EIR auslesen
  //--------------------------------------------------
  if((v_eir_flags&0x01)!=0) {
    // wenn fehler beim empfang
    ENC28J60.rx_error++;
    
    // Error Flag loeschen
    wert=0;
    wert|=0x01; // RXERIF Bit loeschen
    ENC_clearBitRegister(ENC28J60_CRE_EIR,wert);
  }

  //--------------------------------------------------
  // check ob eine Antwort gesendet werden muss
  //--------------------------------------------------
  if(TX_ETH_Data.tx_typ!=ETH_TX_NOTHING) {
    ETH_executeAntwort();
  }

  return(ret_wert);
}

//-----------------------------------------------------------------------------
bool ETH_checkStatusVector(void) {
  // auswerten des RX-Status-Vectors
  // Datasheet Seite 43
  //
  // true, wenn alles ok

  bool ret_wert=true;

  // Bit 31 = egal
  if((RX_ETH_Data.status&0x4000) !=0) ret_wert=false; // Bit 30 (Frame=VLAN)
  // Bit 29-24 = egal
  if((RX_ETH_Data.status&0x0080) ==0) ret_wert=false; // Bit 23 (kein Received Ok)
  if((RX_ETH_Data.status&0x0040) ==0) ret_wert=false; // Bit 22 (kein Typfield)
  if((RX_ETH_Data.status&0x0020) !=0) ret_wert=false; // Bit 21 (Length-Check-Error)
  if((RX_ETH_Data.status&0x0010) !=0) ret_wert=false; // Bit 20 (CRC-Error)
  // Bit 19-16 = egal

  return(ret_wert);
}

//-----------------------------------------------------------------------------
unsigned int ETH_makeChecksum(unsigned int start, unsigned int laenge) {
  // erstellt die Checksumme von einem Teil des Ethernet-Frames
  //
  // start = start adresse im Frame fuer die Checksumme
  // laenge = laenge des teils fuer die Checksumme (muss eine gerade zahl sein)

  unsigned int ret_wert=0,uint;
  unsigned int n;
  unsigned char hi,lo;
  unsigned long v_crc;
  unsigned int uint_lo,uint_hi; 

  // alle Werte addieren
  v_crc=0;
  for(n=0;n<laenge;n+=2) {
    // immer 16 Bit addieren
    hi=RX_ETH_Data.raw_data[start+n+0];
	lo=RX_ETH_Data.raw_data[start+n+1];
    uint=(hi<<8);
    uint=(uint|lo);
    v_crc+=uint;
  }
  
  // komplementbildung
  uint_lo=(v_crc&0x0000FFFF);
  uint_hi=(v_crc>>16);
  v_crc=(uint_lo+uint_hi);
  uint_lo=(v_crc&0x0000FFFF);
  uint_hi=(v_crc>>16);
  v_crc=(uint_lo+uint_hi);
  // in 16bit wandeln
  ret_wert=~(v_crc&0x0000FFFF);

  return(ret_wert);
}

#if USE_ETH_INTMODE==1
//-----------------------------------------------------------------------------
ISR (PCINT0_vect) { // ISR : PinChange 0-7 Interrupt   // INTERNE-Funktion
  // dieser Interrupt wird
  // vom ENC28J60 ausgeloest
  // moegliche Quellen : Receive_Packet oder Receive_Error
  //
  // nach dem Interrupt werden alle Packete im RX-Puffer
  // ausgelesen
  //-------------------------------------------------------------
 
  unsigned char wert;
  ETH_RX_STAT_t check_packet=ETH_RX_OK;

  wert=MK3_PIN_ETH;
  if((wert&ENC28J60_INT)==0) {
    // bei einem Lo-Pegel vom PCINT5-Pin5


    // alle packete abholen
    while(check_packet!=ETH_RX_NO_PACKET) {
      // solange noch packete da sind
      check_packet=ETH_RX_Packet();
    }
  }
}
#endif


#endif  // Ende von #define UB_ETH_ETH


