//-----------------------------------------------------------------------------
// ETH_LO_OSI2_ARP
// Version 3.0
// 17.12.2011 / UB
// "www.Mikrocontroller-4U.de"
// mc-4u(@)t-online.de
//-----------------------------------------------------------------------------
// Lo-Level Funktionen
// mit dem Ethernet-IC : ENC28J60
//
// Funktionen : OSI-Schicht-2 implementation
//              ARP-Dataframe (ETH-Typfield = 0x0806)
//              (wird von OSI-Schicht-1 (ETH) aufgerufen)
//
//-----------------------------------------------------------------------------
// das File muss im Ordner
// "myAVR/WinAVR/AVR/Include/UB_EXT_MODULE/UB_ETH_LOLEVEL/"
// liegen und mit #include <UB_ETH_LOLEVEL/ETH_LO_OSI2_ARP.h>
// eingebunden werden
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
//
// ARP-Packet (Komplett)
// ----------------------
// ARP-Laenge     = 28 Bytes
// Start im Array = ab Adresse 14 (0 bis 13 = ETH-Header)
// Packetlaenge   = 42 Bytes (ETH=14 + ARP=28)
//
// ARP-Packet (Header)
// --------------------
// Laenge         = 8 Bytes
// Adr 14 - 15    = Hardware Adr-Typ (2 Bytes) [FIX 0x0001]
// Adr 16 - 17    = Protokoll Adr-Typ (2 Bytes) [FIX 0x0800]
// Adr 18         = HW-Adr-Size (1 Byte) [FIX 0x06]
// Adr 19         = Protokoll-Adr-Size (1 Byte) [FIX 0x04]
// Adr 20 - 21    = Protokoll-Typ (2 Bytes) [0x0001 oder 0x0002] [Request oder Reply]
//
// ARP-Packet (Daten)
// -------------------
// Laenge         = 20 Bytes
// Adr 22 - 27    = Quell MAC-Adr (6 Bytes)
// Adr 28 - 31    = Quell IP-Adr (4 Bytes)
// Adr 32 - 37    = Ziel MAC-Adr (6 Bytes)
// Adr 38 - 41    = Ziel IP-Adr (4 Bytes)
//
// im Anschluss an den ARP-Dataframe folgen
// 22 Fuellbytes um die minimum Frame Laenge von 64 Bytes zu fuellen
//-----------------------------------------------------------------------------


#ifndef UB_ETH_ARP    // Wichtig !!
#define UB_ETH_ARP    // damit wird das File nur einmal eingebunden

//----------------------------------------------------------------------------
// Includes
//----------------------------------------------------------------------------
// keine



//-----------------------------------------------------------------------------
// Enumerationen fuer die Funktioen
//-----------------------------------------------------------------------------
// siehe ETH_LO_GLOBAL.h

//-----------------------------------------------------------------------------
// Funktionen die von extern benutzt werden koennen
//-----------------------------------------------------------------------------
bool ARP_executeARPPacket(void);
void ARP_makeARPRequest(void);


//-----------------------------------------------------------------------------
// interne Funktionen
//-----------------------------------------------------------------------------
bool ARP_checkBroadcast(void);
bool ARP_checkOwnIP(void);
bool ARP_checkOwnMAC(void);
ETH_ARP_TYP_t ARP_checkARPTyp(void);
void ARP_readARPData(void);
void ARP_makeARPReply(void);



//-----------------------------------------------------------------------------
// interne Defines
//-----------------------------------------------------------------------------
#define  ARP_FRAME_LEN       42 // gesammtlaenge (ETH-Header + ARP-Daten)
#define  ARP_PACKET_LEN      60 // laenge des ARP-Packets (ohne die Checksumme)

#define  ARP_POS_OPERATION_L 21 // position vom ARP-Protokoll-Typ (Lo-Byte)
#define  ARP_POS_Q_IP_ADR    28 // position der Quell-IP-Adr (4 Bytes)
#define  ARP_POS_Z_IP_ADR    38 // position der Ziel-IP-Adr (4 Bytes)


//-----------------------------------------------------------------------------
bool ARP_executeARPPacket(void) {
  // barbeiten von ARP-Packeten
  //
  // rueckgabe von true, wenn Packet bearbeitet wurde
  // setzt u.U. "TX_ETH_Data.tx_typ"

  bool ret_wert=false;
  ETH_ARP_TYP_t arp_typ=ETH_ARP_UNKNWON;


  // pruefen auf ARP-Protokoll-Typ
  arp_typ=ARP_checkARPTyp();
  if(arp_typ==ETH_ARP_REQUEST) {
    // wenn es eine ARP-Anforderung war
    // pruefen auf Broadcast  
    if(ARP_checkBroadcast()==true) { 
      // pruefen auf eigene IP
      if(ARP_checkOwnIP()==true) {
        // wenn die ARP-Request fuer mich
        // alle notwendigen Daten retten
        ARP_readARPData();		
        // und die ARP-Antwort erstellen
        ARP_makeARPReply();
        // damit eine Antwort gesendet wird
        TX_ETH_Data.tx_typ=ETH_TX_ARP_REPLY;
        // TRUE zurueckgeben
        ret_wert=true;
      }
    }
  }
  else if(arp_typ==ETH_ARP_REPLY) {
    // wenn es eine ARP-Antwort war
    // pruefen auf eigene MAC
    if(ARP_checkOwnMAC()==true) {
      // pruefen auf eigene IP
      if(ARP_checkOwnIP()==true) {
        // wenn die ARP-Reply fuer mich
        // alle notwendigen Daten retten
        ARP_readARPData();
        // keine Antwort notwendig
        // TRUE zurueckgeben
        ret_wert=true;
      }
    }
  }

  return(ret_wert);
}

//-----------------------------------------------------------------------------
bool ARP_checkBroadcast(void) {
  // prueft ob die Ziel-MAC-Adresse eine Broadcast
  // = "FF:FF:FF:FF:FF:FF" ist

  bool ret_wert=true;

  if(RX_ETH_Data.z_mac[0]!=0xFF) ret_wert=false;
  if(RX_ETH_Data.z_mac[1]!=0xFF) ret_wert=false;
  if(RX_ETH_Data.z_mac[2]!=0xFF) ret_wert=false;
  if(RX_ETH_Data.z_mac[3]!=0xFF) ret_wert=false;
  if(RX_ETH_Data.z_mac[4]!=0xFF) ret_wert=false;
  if(RX_ETH_Data.z_mac[5]!=0xFF) ret_wert=false;

  return(ret_wert);
}

//-----------------------------------------------------------------------------
bool ARP_checkOwnIP(void) {
  // prueft ob die Ziel-IP Adresse
  // die eigene ist

  bool ret_wert=true;
 
  if(RX_ETH_Data.raw_data[ARP_POS_Z_IP_ADR+0]!=ETH_Setting.own_ip[0]) ret_wert=false;
  if(RX_ETH_Data.raw_data[ARP_POS_Z_IP_ADR+1]!=ETH_Setting.own_ip[1]) ret_wert=false;
  if(RX_ETH_Data.raw_data[ARP_POS_Z_IP_ADR+2]!=ETH_Setting.own_ip[2]) ret_wert=false;
  if(RX_ETH_Data.raw_data[ARP_POS_Z_IP_ADR+3]!=ETH_Setting.own_ip[3]) ret_wert=false;

  return(ret_wert);
}

//-----------------------------------------------------------------------------
bool ARP_checkOwnMAC(void) {
  // prueft ob die Ziel-MAC-Adresse
  // die eigene ist

  bool ret_wert=true;

  if(RX_ETH_Data.z_mac[0]!=ETH_Setting.own_mac[0]) ret_wert=false;
  if(RX_ETH_Data.z_mac[1]!=ETH_Setting.own_mac[1]) ret_wert=false;
  if(RX_ETH_Data.z_mac[2]!=ETH_Setting.own_mac[2]) ret_wert=false;
  if(RX_ETH_Data.z_mac[3]!=ETH_Setting.own_mac[3]) ret_wert=false;
  if(RX_ETH_Data.z_mac[4]!=ETH_Setting.own_mac[4]) ret_wert=false;
  if(RX_ETH_Data.z_mac[5]!=ETH_Setting.own_mac[5]) ret_wert=false;

  return(ret_wert);
}

//-----------------------------------------------------------------------------
ETH_ARP_TYP_t ARP_checkARPTyp(void) {
  // prueft welcher ARP-Protokoll-Typ es ist
  // ARP-Anforderung (request) = 0x01
  // ARP-Antwort (reply) = 0x02

  ETH_ARP_TYP_t ret_wert=ETH_ARP_UNKNWON;

  if(RX_ETH_Data.raw_data[ARP_POS_OPERATION_L]==0x01) ret_wert=ETH_ARP_REQUEST;
  if(RX_ETH_Data.raw_data[ARP_POS_OPERATION_L]==0x02) ret_wert=ETH_ARP_REPLY;

  return(ret_wert);
}

//-----------------------------------------------------------------------------
void ARP_readARPData(void) {
  // alle notwendigen Daten
  // vom ARP-Protokoll speichern

  // Quell-MAC-Adresse wurde schon gespeichert

  // Quell-IP-Adresse speichern
  HOST_ETH_Data.host_ip[0]=RX_ETH_Data.raw_data[ARP_POS_Q_IP_ADR+0];
  HOST_ETH_Data.host_ip[1]=RX_ETH_Data.raw_data[ARP_POS_Q_IP_ADR+1];
  HOST_ETH_Data.host_ip[2]=RX_ETH_Data.raw_data[ARP_POS_Q_IP_ADR+2];
  HOST_ETH_Data.host_ip[3]=RX_ETH_Data.raw_data[ARP_POS_Q_IP_ADR+3];
}

//-----------------------------------------------------------------------------
void ARP_makeARPReply(void) {
  // ARP-Reply-Packet erstellen
  unsigned int n;

  //---------------------------
  // ETH-Header
  //---------------------------
  // Ziel MAC-Adresse
  RX_ETH_Data.raw_data[0]=HOST_ETH_Data.host_mac[0];
  RX_ETH_Data.raw_data[1]=HOST_ETH_Data.host_mac[1];
  RX_ETH_Data.raw_data[2]=HOST_ETH_Data.host_mac[2];
  RX_ETH_Data.raw_data[3]=HOST_ETH_Data.host_mac[3];
  RX_ETH_Data.raw_data[4]=HOST_ETH_Data.host_mac[4];
  RX_ETH_Data.raw_data[5]=HOST_ETH_Data.host_mac[5];

  // Quell MAC-Adresse
  RX_ETH_Data.raw_data[6]=ETH_Setting.own_mac[0];
  RX_ETH_Data.raw_data[7]=ETH_Setting.own_mac[1];
  RX_ETH_Data.raw_data[8]=ETH_Setting.own_mac[2];
  RX_ETH_Data.raw_data[9]=ETH_Setting.own_mac[3];
  RX_ETH_Data.raw_data[10]=ETH_Setting.own_mac[4];
  RX_ETH_Data.raw_data[11]=ETH_Setting.own_mac[5];

  // Typfield (ARP-Packet = 0x0806)
  RX_ETH_Data.raw_data[12]=0x08;
  RX_ETH_Data.raw_data[13]=0x06;

  //---------------------------
  // ARP-Header
  //---------------------------
  // Hardware Adr-Typ (0x0001)
  RX_ETH_Data.raw_data[14]=0x00;
  RX_ETH_Data.raw_data[15]=0x01;

  // Protokoll Adr-Typ (0x0800)
  RX_ETH_Data.raw_data[16]=0x08;
  RX_ETH_Data.raw_data[17]=0x00;

  // HW-Adr-Size (0x06)
  RX_ETH_Data.raw_data[18]=0x06;

  // Protokoll-Adr-Size (0x04)
  RX_ETH_Data.raw_data[19]=0x04;

  // ARP-Protokoll-Typ (ARP-Reply = 0x0002)
  RX_ETH_Data.raw_data[20]=0x00;
  RX_ETH_Data.raw_data[21]=0x02;

  //---------------------------
  // ARP-Daten
  //---------------------------
  // Quell MAC-Adresse
  RX_ETH_Data.raw_data[22]=ETH_Setting.own_mac[0];
  RX_ETH_Data.raw_data[23]=ETH_Setting.own_mac[1];
  RX_ETH_Data.raw_data[24]=ETH_Setting.own_mac[2];
  RX_ETH_Data.raw_data[25]=ETH_Setting.own_mac[3];
  RX_ETH_Data.raw_data[26]=ETH_Setting.own_mac[4];
  RX_ETH_Data.raw_data[27]=ETH_Setting.own_mac[5];

  // Quell IP-Adresse
  RX_ETH_Data.raw_data[28]=ETH_Setting.own_ip[0];
  RX_ETH_Data.raw_data[29]=ETH_Setting.own_ip[1];
  RX_ETH_Data.raw_data[30]=ETH_Setting.own_ip[2];
  RX_ETH_Data.raw_data[31]=ETH_Setting.own_ip[3];

  // Ziel MAC-Adresse
  RX_ETH_Data.raw_data[32]=HOST_ETH_Data.host_mac[0];
  RX_ETH_Data.raw_data[33]=HOST_ETH_Data.host_mac[1];
  RX_ETH_Data.raw_data[34]=HOST_ETH_Data.host_mac[2];
  RX_ETH_Data.raw_data[35]=HOST_ETH_Data.host_mac[3];
  RX_ETH_Data.raw_data[36]=HOST_ETH_Data.host_mac[4];
  RX_ETH_Data.raw_data[37]=HOST_ETH_Data.host_mac[5];

  // Ziel IP-Adresse
  RX_ETH_Data.raw_data[38]=HOST_ETH_Data.host_ip[0];
  RX_ETH_Data.raw_data[39]=HOST_ETH_Data.host_ip[1];
  RX_ETH_Data.raw_data[40]=HOST_ETH_Data.host_ip[2];
  RX_ETH_Data.raw_data[41]=HOST_ETH_Data.host_ip[3];

  // den Rest des Frames mit 0x00 fuellen
  for(n=42;n<ARP_PACKET_LEN;n++) {
    RX_ETH_Data.raw_data[n]=0x00;
  }

  // laenge festlegen
  TX_ETH_Data.eth_tx_len=ARP_PACKET_LEN;

  //-----------------------------------------
  // die ETH-CRC wird vom EN28J60 angehaengt
  //-----------------------------------------
}

//-----------------------------------------------------------------------------
void ARP_makeARPRequest(void) {
  // ARP-Request-Packet erstellen
  unsigned int n;

  //---------------------------
  // ETH-Header
  //---------------------------
  // Ziel MAC-Adresse (Broadcast)
  RX_ETH_Data.raw_data[0]=0xFF;
  RX_ETH_Data.raw_data[1]=0xFF;
  RX_ETH_Data.raw_data[2]=0xFF;
  RX_ETH_Data.raw_data[3]=0xFF;
  RX_ETH_Data.raw_data[4]=0xFF;
  RX_ETH_Data.raw_data[5]=0xFF;

  // Quell MAC-Adresse
  RX_ETH_Data.raw_data[6]=ETH_Setting.own_mac[0];
  RX_ETH_Data.raw_data[7]=ETH_Setting.own_mac[1];
  RX_ETH_Data.raw_data[8]=ETH_Setting.own_mac[2];
  RX_ETH_Data.raw_data[9]=ETH_Setting.own_mac[3];
  RX_ETH_Data.raw_data[10]=ETH_Setting.own_mac[4];
  RX_ETH_Data.raw_data[11]=ETH_Setting.own_mac[5];

  // Typfield (ARP-Packet = 0x0806)
  RX_ETH_Data.raw_data[12]=0x08;
  RX_ETH_Data.raw_data[13]=0x06;

  //---------------------------
  // ARP-Header
  //---------------------------
  // Hardware Adr-Typ (0x0001)
  RX_ETH_Data.raw_data[14]=0x00;
  RX_ETH_Data.raw_data[15]=0x01;

  // Protokoll Adr-Typ (0x0800)
  RX_ETH_Data.raw_data[16]=0x08;
  RX_ETH_Data.raw_data[17]=0x00;

  // HW-Adr-Size (0x06)
  RX_ETH_Data.raw_data[18]=0x06;

  // Protokoll-Adr-Size (0x04)
  RX_ETH_Data.raw_data[19]=0x04;

  // ARP-Protokoll-Typ (ARP-Request = 0x0001)
  RX_ETH_Data.raw_data[20]=0x00;
  RX_ETH_Data.raw_data[21]=0x01;

  //---------------------------
  // ARP-Daten
  //---------------------------
  // Quell MAC-Adresse
  RX_ETH_Data.raw_data[22]=ETH_Setting.own_mac[0];
  RX_ETH_Data.raw_data[23]=ETH_Setting.own_mac[1];
  RX_ETH_Data.raw_data[24]=ETH_Setting.own_mac[2];
  RX_ETH_Data.raw_data[25]=ETH_Setting.own_mac[3];
  RX_ETH_Data.raw_data[26]=ETH_Setting.own_mac[4];
  RX_ETH_Data.raw_data[27]=ETH_Setting.own_mac[5];

  // Quell IP-Adresse
  RX_ETH_Data.raw_data[28]=ETH_Setting.own_ip[0];
  RX_ETH_Data.raw_data[29]=ETH_Setting.own_ip[1];
  RX_ETH_Data.raw_data[30]=ETH_Setting.own_ip[2];
  RX_ETH_Data.raw_data[31]=ETH_Setting.own_ip[3];

  // Ziel MAC-Adresse (unbenutzt)
  RX_ETH_Data.raw_data[32]=0x00;
  RX_ETH_Data.raw_data[33]=0x00;
  RX_ETH_Data.raw_data[34]=0x00;
  RX_ETH_Data.raw_data[35]=0x00;
  RX_ETH_Data.raw_data[36]=0x00;
  RX_ETH_Data.raw_data[37]=0x00;

  // Ziel IP-Adresse
  RX_ETH_Data.raw_data[38]=HOST_ETH_Data.host_ip[0];
  RX_ETH_Data.raw_data[39]=HOST_ETH_Data.host_ip[1];
  RX_ETH_Data.raw_data[40]=HOST_ETH_Data.host_ip[2];
  RX_ETH_Data.raw_data[41]=HOST_ETH_Data.host_ip[3];

  // den Rest des Frames mit 0x00 fuellen
  for(n=42;n<ARP_PACKET_LEN;n++) {
    RX_ETH_Data.raw_data[n]=0x00;
  }

  // laenge festlegen
  TX_ETH_Data.eth_tx_len=ARP_PACKET_LEN;

  //-----------------------------------------
  // die ETH-CRC wird vom EN28J60 angehaengt
  //-----------------------------------------
}


#endif  // Ende von #define UB_ETH_ARP


