//-----------------------------------------------------------------------------
// ETH_LO_OSI3_IP
// Version 3.0
// 17.12.2011 / UB
// "www.Mikrocontroller-4U.de"
// mc-4u(@)t-online.de
//-----------------------------------------------------------------------------
// Lo-Level Funktionen
// mit dem Ethernet-IC : ENC28J60
//
// Funktionen : OSI-Schicht-3 implementation
//              IP-Dataframe (ETH-Typfield = 0x0800)
//              (wird von OSI-Schicht-1 (ETH) aufgerufen)
//
//-----------------------------------------------------------------------------
// das File muss im Ordner
// "myAVR/WinAVR/AVR/Include/UB_EXT_MODULE/UB_ETH_LOLEVEL/"
// liegen und mit #include <UB_ETH_LOLEVEL/ETH_LO_OSI3_IP.h>
// eingebunden werden
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
//
// IP-Packet (Komplett)
// ----------------------
// IP-Laenge      = xx Bytes
// Start im Array = ab Adresse 14 (0 bis 13 = ETH-Header)
// Packetlaenge   = xx Bytes (ETH=14 + IP=xx)
//
// IP-Packet (Header)
// ----------------------
// Laenge         = 20 Bytes
// Adr 14         = Version und IHL (1 Byte)
// Adr 15         = Typ of Service (1 byte)
// Adr 16 - 17    = Laenge (2 Bytes)
// Adr 18 - 19    = Identifikation (2 Bytes)
// Adr 20 - 21    = Flag und Offset (2 Bytes)
// Adr 22         = TTL (1 Byte)
// Adr 23         = Protokoll (1 Byte) [ICMP, UDP usw]
// Adr 24 - 25    = IP-Chksum (2 Bytes)
// Adr 26 - 29    = Quell IP-Adr (4 Bytes)
// Adr 30 - 33    = Ziel IP-Adr (4 Bytes)
//
// IP-Packet (Daten)
// ----------------------
// im Anschluss an den IP-Header folgt
// der Frame des jeweiligen Typs (z.B. ICMP oder UDP usw)
//-----------------------------------------------------------------------------


#ifndef UB_ETH_IP    // Wichtig !!
#define UB_ETH_IP    // damit wird das File nur einmal eingebunden


//----------------------------------------------------------------------------
// Includes
//----------------------------------------------------------------------------
#include "ETH_LO_OSI3_ICMP.h" // fuer ICMP-Protokolle
#include "ETH_LO_OSI4_UDP.h"  // fuer UDP-Protokolle


//-----------------------------------------------------------------------------
// Enumerationen fuer die Funktioen
//-----------------------------------------------------------------------------
// siehe ETH_LO_GLOBAL.h


//-----------------------------------------------------------------------------
// Funktionen die von extern benutzt werden koennen
//-----------------------------------------------------------------------------
bool IP_executeIPPacket(void);


//-----------------------------------------------------------------------------
// interne Funktionen
//-----------------------------------------------------------------------------
bool IP_checkOwnMAC(void);
bool IP_checkOwnIP(void);
ETH_IP_TYP_t IP_checkIPTyp(void);
void IP_readIPData(void);
void IP_makeIPChecksum(void);



//-----------------------------------------------------------------------------
// interne Defines
//-----------------------------------------------------------------------------
#define  IP_HEADER_LEN      20 // Headerlaenge (IP-Header)
#define  IP_POS_HEADER      14 // start vom IP-Header (20 Bytes)
#define  IP_POS_LEN         16 // position der Laenge (2 Bytes)
#define  IP_POS_TYP         23 // position vom IP-Protokoll-Typ (1 Byte)
#define  IP_POS_CRC_HI      24 // position der CRC (Hi-Byte)
#define  IP_POS_CRC_LO      25 // position der CRC (Lo-Byte)
#define  IP_POS_Q_IP_ADR    26 // position der Quell-IP-Adr (4 Bytes)
#define  IP_POS_Z_IP_ADR    30 // position der Ziel-IP-Adr (4 Bytes)



//-----------------------------------------------------------------------------
bool IP_executeIPPacket(void)
{
  // barbeiten von IP-Packeten
  //
  // rueckgabe von true, wenn Packet bearbeitet wurde

  bool ret_wert=false;
  ETH_IP_TYP_t ip_typ=ETH_IP_UNKNOWN;

  // pruefen auf IP-Protokoll-Typ
  ip_typ=IP_checkIPTyp();
  if(ip_typ==ETH_IP_ICMP) {
    // wenn es ein ICMP-Packet war
    // pruefen auf eigene MAC
    if(IP_checkOwnMAC()==true) {
      // pruefen auf eigene IP
      if(IP_checkOwnIP()==true) {
        // wenn das ICMP-Packet fr mich
        // alle notwendigen Daten retten
        IP_readIPData();
        // laenge ueberpruefen
        if(RX_ETH_Data.eth_frame_len<=ETH_MAX_ICMP_SIZE) {
          // das ICMP-Packet bearbeiten
          ret_wert=ICMP_executeICMPPacket();
          if(TX_ETH_Data.tx_typ==ETH_TX_ICMP_REPLY) {
            // wenn eine ICMP-Antwort gesendet werden soll
	    // IP-Checksumme hinzufuegen
            IP_makeIPChecksum();
          }
        }
      }
    }
  }
  else if(ip_typ==ETH_IP_UDP) {
    // wenn es ein UDP-Packet war
    // pruefen auf eigene MAC
    if(IP_checkOwnMAC()==true) {
      // pruefen auf eigene IP
      if(IP_checkOwnIP()==true) {
        // wenn das UDP-Packet fr mich
        // alle notwendigen Daten retten
        IP_readIPData();
        // laenge ueberpruefen
        if(RX_ETH_Data.eth_frame_len<=ETH_MAX_UDP_SIZE) {
          // das UDP-Packet bearbeiten
          ret_wert=UDP_executeUDPPacket();
          if(TX_ETH_Data.tx_typ==ETH_TX_UDP_PACKET) {
            // wenn ein UDP-Packet gesendet werden soll
	    // IP-Checksumme hinzufuegen
            IP_makeIPChecksum();
          }
        }
      }
    }
  }
  

  return(ret_wert);
}


//-----------------------------------------------------------------------------
bool IP_checkOwnMAC(void) {
  // prueft ob die Ziel-MAC-Adresse
  // die eigene ist

  bool ret_wert=true;

  if(RX_ETH_Data.z_mac[0]!=ETH_Setting.own_mac[0]) ret_wert=false;
  if(RX_ETH_Data.z_mac[1]!=ETH_Setting.own_mac[1]) ret_wert=false;
  if(RX_ETH_Data.z_mac[2]!=ETH_Setting.own_mac[2]) ret_wert=false;
  if(RX_ETH_Data.z_mac[3]!=ETH_Setting.own_mac[3]) ret_wert=false;
  if(RX_ETH_Data.z_mac[4]!=ETH_Setting.own_mac[4]) ret_wert=false;
  if(RX_ETH_Data.z_mac[5]!=ETH_Setting.own_mac[5]) ret_wert=false;

  return(ret_wert);
}

//-----------------------------------------------------------------------------
bool IP_checkOwnIP(void) {
  // prueft ob die Ziel-IP Adresse
  // die eigene ist

  bool ret_wert=true;
 
  if(RX_ETH_Data.raw_data[IP_POS_Z_IP_ADR+0]!=ETH_Setting.own_ip[0]) ret_wert=false;
  if(RX_ETH_Data.raw_data[IP_POS_Z_IP_ADR+1]!=ETH_Setting.own_ip[1]) ret_wert=false;
  if(RX_ETH_Data.raw_data[IP_POS_Z_IP_ADR+2]!=ETH_Setting.own_ip[2]) ret_wert=false;
  if(RX_ETH_Data.raw_data[IP_POS_Z_IP_ADR+3]!=ETH_Setting.own_ip[3]) ret_wert=false;

  return(ret_wert);
}

//-----------------------------------------------------------------------------
ETH_IP_TYP_t IP_checkIPTyp(void) {
  // prueft welcher IP-Protokoll-Typ es ist
  // ICMP = 0x01
  // UDP  = 0x11
  
  ETH_IP_TYP_t ret_wert=ETH_IP_UNKNOWN;

  if(RX_ETH_Data.raw_data[IP_POS_TYP]==0x01) ret_wert=ETH_IP_ICMP;
  if(RX_ETH_Data.raw_data[IP_POS_TYP]==0x11) ret_wert=ETH_IP_UDP;
  
  return(ret_wert); 
}

//-----------------------------------------------------------------------------
void IP_readIPData(void) {
  // alle notwendigen Daten
  // vom IP-Protokoll speichern

  unsigned int uint=0;
  unsigned char lo,hi;

  // Quell-MAC-Adresse wurde schon gespeichert

  // Quell-IP-Adresse speichern
  HOST_ETH_Data.host_ip[0]=RX_ETH_Data.raw_data[IP_POS_Q_IP_ADR+0];
  HOST_ETH_Data.host_ip[1]=RX_ETH_Data.raw_data[IP_POS_Q_IP_ADR+1];
  HOST_ETH_Data.host_ip[2]=RX_ETH_Data.raw_data[IP_POS_Q_IP_ADR+2];
  HOST_ETH_Data.host_ip[3]=RX_ETH_Data.raw_data[IP_POS_Q_IP_ADR+3];

  // laenge vom IP-Frame auslesen
  hi=RX_ETH_Data.raw_data[IP_POS_LEN+0];
  lo=RX_ETH_Data.raw_data[IP_POS_LEN+1];
  uint=(hi<<8);
  uint=(uint|lo);
  RX_ETH_Data.ip_frame_len=uint;
  // Header laenge abziehen
  // und laenge der IP-Daten speichern
  RX_ETH_Data.ip_data_len=uint-IP_HEADER_LEN;
}

//-----------------------------------------------------------------------------
void IP_makeIPChecksum(void) {
  // fuegt die IP-Checksumme in den Frame ein

  unsigned int ip_crc=0;
  unsigned char lo,hi;

  // crc erstellen ueber den IP-Header
  ip_crc=ETH_makeChecksum(IP_POS_HEADER,IP_HEADER_LEN);

  lo=(ip_crc&0x00FF);
  hi=(ip_crc>>8);

  // crc im Frame speichern
  RX_ETH_Data.raw_data[IP_POS_CRC_HI]=hi;
  RX_ETH_Data.raw_data[IP_POS_CRC_LO]=lo;
}



#endif  // Ende von #define UB_ETH_IP


